/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.api;

import java.io.InputStream;
import java.util.Properties;

import javax.security.auth.login.LoginContext;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

/*
 * AgreementFactoryLocator
 */

/**
 * The Locator returns a new wsag4j client instances. The lookup mechanism for a client implementation class
 * is as follows:
 * <ol>
 * <li>load the class specified in the system property <code>client interface class name</code>, for example
 * {@linkplain org.ogf.graap.wsag.client.api.AgreementFactoryRegistryClient}</li>
 * <li>load the class specified in the {@value #APP_CONFIG_FILENAME} file</li>
 * <li>load the class specified in the {@value #IMPLEMENTATION_CONFIG_FILENAME} file</li>
 * </ol>
 * 
 * The key used in the configuration (system property) corresponds to the name of the client interface.
 * 
 * 
 * @param <T>
 *            the type of the client
 * 
 * @author Oliver Waeldrich
 * 
 */
public class ClientLocator<T>
{
    /**
     * client implementation configuration
     */
    private static final String IMPLEMENTATION_CONFIG_FILENAME = "/META-INF"
        + ClientFactory.DEFAULT_CONFIGURATION_FILE;

    /**
     * application configuration
     */
    private static final String APP_CONFIG_FILENAME = ClientFactory.DEFAULT_CONFIGURATION_FILE;

    /**
     * The name of the client interface class.
     */
    private final String configurationKey;

    private final String implementationClassFromConfig;

    /**
     * Instantiates a new client locator.
     * 
     * @param clazz
     *            the client class
     */
    protected ClientLocator( Class<T> clazz )
    {
        configurationKey = clazz.getName();
        implementationClassFromConfig = readClassFromConfig();
    }

    private synchronized ClientFactory<T> newClientFactoryInstance()
    {
        return newClientFactoryInstance( null );
    }

    private synchronized ClientFactory<T> newClientFactoryInstance( String alternativeConfiguration )
    {
        //
        // read from system properties
        //
        String clazzName =
            System.getProperties().getProperty( configurationKey, implementationClassFromConfig );

        if ( alternativeConfiguration != null )
        {
            clazzName = readClassFromConfigFile( alternativeConfiguration, clazzName );
        }

        if ( ( clazzName == null ) || "".equals( clazzName ) )
        {
            throw new RuntimeException( "failed to lookup client factory implementation for interface"
                + configurationKey );
        }

        //
        // instantiate locator class
        //
        try
        {
            @SuppressWarnings( "unchecked" )
            Class<ClientFactory<T>> clazz =
                (Class<ClientFactory<T>>) Class.forName( clazzName, true,
                    ClientLocator.class.getClassLoader() );

            ClientFactory<T> instance = clazz.newInstance();
            return instance;
        }
        catch ( Exception e )
        {
            throw new RuntimeException( "failed to instantiate client factory implementation for interface "
                + configurationKey );
        }
        catch ( Error e )
        {
            System.err.println( "failed to instantiate client factory implementation for interface "
                + configurationKey );
            System.err.println( e.getMessage() );
            throw e;
        }
    }

    private String readClassFromConfig()
    {
        //
        // read from client implementation configuration, no default value
        //
        String clientImplConfig = readClassFromConfigFile( IMPLEMENTATION_CONFIG_FILENAME, null );

        //
        // read from application configuration, default value from client impl configuration
        //
        String appConfig = readClassFromConfigFile( APP_CONFIG_FILENAME, clientImplConfig );
        return appConfig;
    }

    /**
     * Returns the locator class name form the wsag4j-client.config file at he specified path. If the property
     * does not exist the default value is returned.
     * 
     * @return the locator class name
     */
    private String readClassFromConfigFile( String fileName, String defaultValue )
    {
        Properties properties = new Properties();
        try
        {
            InputStream inputStream = ClientLocator.class.getResourceAsStream( fileName );
            properties.load( inputStream );
        }
        catch ( Exception e )
        {
            // failed to lookup config file
        }

        String fromConfig = properties.getProperty( configurationKey, defaultValue );

        return fromConfig;
    }

    /**
     * Returns a new client instance.
     * 
     * @param epr
     *            the client epr
     * 
     * @return the new client
     * 
     * @throws Exception
     *             client instantiation error
     */
    public T newInstance( EndpointReferenceType epr ) throws Exception
    {
        return newClientFactoryInstance().newInstance( epr );
    }

    /**
     * Returns a new client instance.
     * 
     * @param epr
     *            the client epr
     * 
     * @param context
     *            client login context
     * 
     * @return the new client
     * 
     * @throws Exception
     *             client instantiation error
     */
    public T newInstance( EndpointReferenceType epr, LoginContext context ) throws Exception
    {
        return newClientFactoryInstance().newInstance( epr, context );
    }

    /**
     * Returns a new client instance providing an alternative client configuration file.
     * 
     * @param epr
     *            the client epr
     * 
     * @param altConfig
     *            alternative client factory implementation
     * 
     * @return the new client
     * 
     * @throws Exception
     *             client instantiation error
     */
    protected T newInstance( EndpointReferenceType epr, String altConfig ) throws Exception
    {
        return newClientFactoryInstance( altConfig ).newInstance( epr );
    }

    /**
     * Returns a new client instance providing an alternative client configuration file.
     * 
     * @param epr
     *            the client epr
     * 
     * @param context
     *            client login context
     * 
     * @param altConfig
     *            alternative client factory implementation
     * 
     * @return the new client
     * 
     * @throws Exception
     *             client instantiation error
     */
    protected T newInstance( EndpointReferenceType epr, LoginContext context, String altConfig )
        throws Exception
    {
        return newClientFactoryInstance( altConfig ).newInstance( epr, context );
    }
}
