/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.api.impl;

import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.NegotiationClient;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * @author owaeld
 * 
 */
public abstract class AbstractAgreementFactoryClient extends AbstractClient
    implements AgreementFactoryClient
{
    /**
     * Returns the templates supported by this factory.
     * 
     * {@inheritDoc}
     */
    public abstract AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException;

    /**
     * {@inheritDoc}
     */
    public abstract AgreementFactoryPropertiesType getResourceProperties()
        throws ResourceUnknownException, ResourceUnavailableException;

    /**
     * Creates a new agreement instance.
     * 
     * @param offer
     *            the offer to create the agreement for
     * 
     * @return the created agreement
     * 
     * @throws AgreementFactoryException
     *             the agreement offer was rejected by the factory
     * 
     * @throws ResourceUnavailableException
     *             the requested agreement factory resource is unavailable
     * 
     * @throws ResourceUnknownException
     *             the requested agreement factory resource is unknown
     */
    public abstract AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException;

    /**
     * Creates a new pending agreement instance.
     * 
     * {@inheritDoc}
     */
    public abstract AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException;

    /**
     * Creates a new pending agreement instance.
     * 
     * {@inheritDoc}
     */
    public abstract AgreementClient createPendingAgreement( AgreementOffer offer,
                                                            EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException;

    /**
     * Initiates a Negotiation.
     * 
     * {@inheritDoc}
     */
    public abstract NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException;

    /**
     * {@inheritDoc}
     */
    public abstract String getResourceId() throws ResourceUnknownException, ResourceUnavailableException;

    /**
     * 
     * @return a template with the given name and id, or null if no appropriate template was found.
     * 
     *         {@inheritDoc}
     */
    public AgreementTemplateType getTemplate( String name, String id )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        if ( name == null )
        {
            return null;
        }

        id = ( id != null ) ? id : "";

        AgreementTemplateType[] templates = getTemplates();

        for ( int i = 0; i < templates.length; i++ )
        {
            if ( name.equalsIgnoreCase( templates[i].getName() ) )
            {

                String actualId =
                    ( templates[i].getTemplateId() != null ) ? templates[i].getTemplateId() : "";

                if ( id.equalsIgnoreCase( actualId ) )
                {
                    return templates[i];
                }
            }
        }

        return null;
    }

    /**
     * Creates s copy of this AgreementFactoryClient.
     * 
     * @return the cloned agreement factory client
     * 
     * @throws CloneNotSupportedException
     *             this particular client implementation does not support cloning
     */
    @Override
    public abstract AgreementFactoryClient clone() throws CloneNotSupportedException;

    /**
     * Lists all agreements of this factory.
     * 
     * @return all agreement clients known by this factory
     * 
     * @throws ResourceUnavailableException
     *             the requested agreement factory resource is unavailable
     * 
     * @throws ResourceUnknownException
     *             the requested agreement factory resource is unknown
     */
    public abstract AgreementClient[] listAgreements()
        throws ResourceUnknownException, ResourceUnavailableException;
}
