/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.api.local;

import java.util.HashMap;

import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.PendingAgreementListener;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.NegotiationClient;
import org.ogf.graap.wsag.client.api.RemoteClient;
import org.ogf.graap.wsag.client.api.impl.AbstractAgreementFactoryClient;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * An implementation of the Client API for local invocation of an agreement factory.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class LocalAgreementFactoryClient extends AbstractAgreementFactoryClient
{
    private AgreementClient[] agreements = new AgreementClient[0];

    private final AgreementFactory factory;

    private final RemoteClient remoteClient = new RemoteClient( "http://127.0.0.1/agreement-factory" );

    /**
     * Creates a client for local invocation of an agreement factory.
     * 
     * @param factory
     *            the factory object
     */
    public LocalAgreementFactoryClient( AgreementFactory factory )
    {
        this.factory = factory;
    }

    /**
     * 
     * @param factory
     *            the local factory object
     * @param source
     *            the agreements associated with this registry.
     */
    public LocalAgreementFactoryClient( AgreementFactory factory, Agreement[] source )
    {
        this.factory = factory;

        agreements = new AgreementClient[source.length];
        for ( int i = 0; i < source.length; i++ )
        {
            agreements[i] = new LocalAgreementClient( source[i] );
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementFactoryClient clone() throws CloneNotSupportedException
    {
        return new LocalAgreementFactoryClient( factory );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementClient createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {
        HashMap<String, Object> context = new HashMap<String, Object>();
        return new LocalAgreementClient( factory.createAgreement( offer, context ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementClient createPendingAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {
        HashMap<String, Object> context = new HashMap<String, Object>();
        PendingAgreementListener listener = null;
        return new LocalAgreementClient( factory.createPendingAgreement( offer, listener, context ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getResourceId()
    {
        //
        // TODO: use PersistentAgreementfactory as interface and return configured id for the factory
        //
        return "1";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementTemplateType[] getTemplates()
    {
        return factory.getTemplates();
    }

    @Override
    public AgreementTemplateType getTemplate( String name, String id )
    {
        try
        {
            return super.getTemplate( name, id );
        }
        catch ( Exception e )
        {
            //
            // local call, remote exceptions shouldn't be thrown
            //
            throw new RuntimeException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementClient[] listAgreements()
    {
        return agreements;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.client.impl.AbstractClient#getRemoteClient()
     */
    @Override
    public RemoteClient getRemoteClient()
    {
        return remoteClient;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ogf.graap.wsag.client.impl.AbstractAgreementFactoryClient#createPendingAgreement(org.ogf.graap.
     * wsag.api.AgreementOffer, org.w3.x2005.x08.addressing.EndpointReferenceType)
     */
    @Override
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException
    {
        PendingAgreementListener listener = null;
        HashMap<String, Object> context = new HashMap<String, Object>();

        return new LocalAgreementClient( factory.createPendingAgreement( offer, listener, context ) );
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ogf.graap.wsag.client.impl.AbstractAgreementFactoryClient#initiateNegotiation(org.ogf.schemas.graap
     * .wsAgreement.negotiation.NegotiationContextType)
     */
    @Override
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException
    {
        return new LocalNegotiationClient( factory.initiateNegotiation( context, new XmlObject[0],
            new XmlObject[0], new HashMap<String, Object>() ) );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.client.api.AgreementFactoryClient#getResourceProperties()
     */
    @Override
    public AgreementFactoryPropertiesType getResourceProperties()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementFactoryPropertiesDocument properties =
            AgreementFactoryPropertiesDocument.Factory.newInstance();

        properties.addNewAgreementFactoryProperties().setTemplateArray( getTemplates() );
        return properties.getAgreementFactoryProperties();
    }

}
